using System;
using System.ComponentModel;
using System.Web;
using System.Web.UI;
using System.Web.UI.Design;
using System.Web.UI.WebControls;
using RepPP;
using RepPP.Toolkit.Web;

namespace RepPP.Toolkit.Web {
    //********************************************************************************
    //
    /// <summary>
    /// Implement button which the text is defined by a rep++ atom and is not posting back when pushed
    /// </summary>
	//
    //********************************************************************************
	[ToolboxData("<{0}:AtmButtonNP runat=server></{0}:AtmButtonNP>")]
	public class AtmButtonNP : ButtonNP, AtmControlUtil.IAtmControl {
	    /// <summary>Atom control utility class</summary>
	    protected AtmControlUtil        m_atmControlUtil;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:           Class constructor
        /// </summary>
        //
        //********************************************************************************
        public AtmButtonNP() {
            m_atmControlUtil = new AtmControlUtil(this, true);
        }

        //********************************************************************************
        //
		/// <summary>
		/// Dispose:            Dispose the control
		/// </summary>
		//
        //********************************************************************************
        public override void Dispose() {
            if (m_atmControlUtil != null) {
                m_atmControlUtil.Dispose();
            }
            base.Dispose();
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReInit:        Called when Rep++ is reinitialized in design mode
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReInit() {
        }

        //********************************************************************************
        //
		/// <summary>
		/// RepPPReset:         Called when Rep++ is reset in design mode
		/// </summary>
		//
        //********************************************************************************
        public void RepPPReset() {
        }

        //********************************************************************************
        //
        /// <summary>
        /// AtomFullName:               Full name of the atom attached to the label
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Visible)]
        [PersistenceMode(PersistenceMode.Attribute)]
        [Editor(typeof(AtmControlUtil.AtmControlEditor), typeof(System.Drawing.Design.UITypeEditor))]
        [Browsable(true)]
        [Bindable(true)]
        [Category("Data")]
        [Description("Full name of the atom attached to the label")]
		public AtmFullName AtomFullName {
		    get {
		        return(m_atmControlUtil.AtomFullName);
		    }
		    set {
		        m_atmControlUtil.AtomFullName = value;
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// Application:            Gets the Rep++ application
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [Browsable(false)]
		public Application Application {
		    get {
		        return(m_atmControlUtil.Application);
		    }
		}

        //********************************************************************************
        //
        /// <summary>
        /// Text:               Text of the label
        /// </summary>
        //
        //********************************************************************************
        [DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
        [PersistenceMode(PersistenceMode.InnerDefaultProperty)]
        [Browsable(false)]
        [Bindable(false)]
        public override string Text {
            get {
                string          strRetVal;
                string          strFormatText;
                
                strFormatText = FormattedText;
                strRetVal     = (strFormatText == null || strFormatText.Length == 0) ? m_atmControlUtil.AtomText : strFormatText;
                return(strRetVal);
            }
            set {
                throw new System.NotSupportedException();
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// FormattedText:               Formatted text of the label
        /// </summary>
        //
        //********************************************************************************
        [Bindable(true)]
        [Category("Appearance")]
        [DefaultValue("")]
        [Description("Formatted atom text")]
        public string FormattedText {
            get {
                string strRetVal;
                object obj;
                
                obj = ViewState["FormattedText"];
                if (obj != null) {
                    strRetVal = (string)obj;
                } else {
                    strRetVal = "";
                }
                return(strRetVal);
            }
            set {
                ViewState["FormattedText"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// TriggerComponentChange:     Trigger the event indicating a component property has changed
        /// </summary>
        //
        //********************************************************************************
        public virtual void TriggerComponentChange() {
            if (m_atmControlUtil != null) {
                m_atmControlUtil.TriggerComponentChange();
            }
        }
	} // Class AtmButtonNP
} // Namespace
