using System;
using System.IO;
using System.Collections;
using System.ComponentModel;
using System.Web;
using System.Web.UI;
using System.Web.UI.Design;
using System.Web.UI.WebControls;

namespace RepPP.Toolkit.Web {
    //********************************************************************************
    //
    /// <summary>
    /// ContextMenuLink:    Class used to pen the specified ContextMenu
    /// </summary>
    //
    //********************************************************************************
    [DefaultProperty("ContextMenuToOpen")]
   	[ToolboxItem(false)]
    [Designer(typeof(ContextMenuLink.MenuLinkDesigner))]
    public class ContextMenuLink : System.Web.UI.WebControls.WebControl {
        //********************************************************************************
        //
        /// <summary>
        /// ImageUrl:   Get or set the url of the image to display.
        /// </summary>
        //
        //********************************************************************************
        [Description("Gets or sets the url of the image to display.")]
        [Category("Appearance")]
        [DefaultValue("")]
        [Bindable(true)]
        [EditorAttribute(typeof(System.Web.UI.Design.ImageUrlEditor), typeof(System.Drawing.Design.UITypeEditor))]
        public virtual string ImageUrl {
            get {
                string strRetVal;
                object obj;

                obj         = ViewState["ImageUrl"];
                strRetVal   = (obj != null) ? (string)obj : strRetVal = String.Empty;
                return(strRetVal);
            }
            set {
                ViewState["ImageUrl"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// Text:       Get or set the text to display for this item.
        /// </summary>
        //
        //********************************************************************************
        [Description("Gets or sets the Text of the link.")]
        [Category("Appearance")]
        [DefaultValue("")]
        [Bindable(true)]
        public virtual string Text {
            get {
                string strRetVal;
                object obj;

                obj         = ViewState["Text"];
                strRetVal   = (obj != null) ? (string)obj : String.Empty;
                return(strRetVal);
            }
            set {
                ViewState["Text"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// ContextMenuToOpen:      Get or set the context menu to open.
        /// </summary>
        //
        //********************************************************************************
        [Description("Gets or sets the Id of the ContextMenu to associate with the link.")]
        [Category("Appearance")]
        [DefaultValue("")]
        [Bindable(true)]
        [TypeConverter(typeof(MenuControlConverter))]
        public virtual string ContextMenuToOpen {
            get {
                string strRetVal;
                object obj;

                obj         = ViewState["ContextMenuToOpen"];
                strRetVal   = (obj != null) ? (string)obj : String.Empty;
                return(strRetVal);
            }
            set {
                ViewState["ContextMenuToOpen"] = value;
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// TagKey:     Get the System.Web.UI.HtmlTextWriterTag value that corresponds to 
        ///             this Web server control.
        /// </summary>
        //
        //********************************************************************************
        protected override System.Web.UI.HtmlTextWriterTag TagKey {
            get {
                return(HtmlTextWriterTag.A);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// RenderChildren:         Render the children controls.
        /// </summary>
        /// <param name="writer">   The Html text writer object</param>
        //
        //********************************************************************************
        protected override void RenderChildren(System.Web.UI.HtmlTextWriter writer) {
            System.Web.UI.WebControls.Image image;

            if (ImageUrl != String.Empty) {
                image               = new System.Web.UI.WebControls.Image();
                image.ImageUrl      = ImageUrl;
                image.AlternateText = Text;
                image.RenderControl(writer);
            } else {
                writer.Write(Text);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// AddAttributesToRender:  Add attributes to html text writer.
        /// </summary>
        /// <param name="writer">   The Html text writer object</param>
        //
        //********************************************************************************
        protected override void AddAttributesToRender(System.Web.UI.HtmlTextWriter writer) {
            object menu;

            if (System.Web.HttpContext.Current != null &&
                Page != null                           &&
                Enabled == true                        &&
                ContextMenuToOpen.Length > 0) {
                menu = NamingContainer.FindControl(ContextMenuToOpen);
                if (menu == null) {
                    menu = Page.FindControl(ContextMenuToOpen);
                }
                if (menu == null) {
                    throw new InvalidOperationException("Cannot find a ContextMenu with the id '" + ContextMenuToOpen + "'");
                }
                writer.AddAttribute(HtmlTextWriterAttribute.Href, "javascript:void(0);");
                writer.AddAttribute(HtmlTextWriterAttribute.Onclick, (menu as ContextMenu).GetMenuOpenScript());
            } else {
                writer.AddAttribute(HtmlTextWriterAttribute.Href, "javascript:void(0);");
            }
            base.AddAttributesToRender(writer);
        }

        //********************************************************************************
        //
        /// <summary>
        /// The designer for the dialog opener controls.
        /// </summary>
        //
        //********************************************************************************
        public class MenuLinkDesigner : System.Web.UI.Design.ControlDesigner {
            //********************************************************************************
            //
            /// <summary>
            /// GetDesignTimeHtml:  Get the design time html.
            /// </summary>
            /// <returns>
            /// Html text
            /// </returns>
            //
            //********************************************************************************
            public override string GetDesignTimeHtml() {
                StringWriter    stringWriter;
                HtmlTextWriter  htmlWriter;
                ContextMenuLink link;
                HyperLink       placeholderLink;

                link = (Component as ContextMenuLink);
                if (link.Text.Length > 0 ) {
                    stringWriter                = new StringWriter();
                    htmlWriter                  = new HtmlTextWriter(stringWriter);
                    placeholderLink             = new HyperLink();
                    placeholderLink.Text        = link.Text;
                    placeholderLink.NavigateUrl = link.Text;
                    placeholderLink.RenderControl(htmlWriter);
                    return(stringWriter.ToString());
                } else {
                    return(GetEmptyDesignTimeHtml());
                }
            }
        } // Class MenuLinkDesigner

        //********************************************************************************
        //
        /// <summary>
        /// MenuControlConverter:   Gives better designer support for the ContextMenuToOpen 
        ///                         properties of the ContextMenuLink.
        /// </summary>
		//
        //********************************************************************************
        public class MenuControlConverter : StringConverter {

            //********************************************************************************
            //
		    /// <summary>
		    /// ctor:               Class constructor
		    /// </summary>
		    //
            //********************************************************************************
            public MenuControlConverter() {
            }

            //********************************************************************************
            //
            /// <summary>
            /// GetStandardValuesSupported:     Indicates this type converter provides 
            ///                                 a list of standard values.
            /// </summary>
            /// <param name="context">          context</param>
            /// <returns>
            /// True:       Standard values are supported
            /// False:      Standard values are not supported
            /// </returns>
		    //
            //********************************************************************************
            public override bool GetStandardValuesSupported(ITypeDescriptorContext context) {
                return(true);
            }

            //********************************************************************************
            //
            /// <summary>
            /// GetStandardValues:      Returns a collection of standard value objects.
            /// </summary>
            /// <param name="context">  Context</param>
            /// <returns>
            /// Collection of standard value objects.
            /// </returns>
 		    //
            //********************************************************************************
            public override TypeConverter.StandardValuesCollection GetStandardValues(ITypeDescriptorContext context) {
                TypeConverter.StandardValuesCollection valuesRetVal = null;
                object[] serverControls;

                if (context != null) {
                    serverControls = GetControls(context.Container);
                    if (serverControls != null) {
                        valuesRetVal = new StandardValuesCollection(serverControls);
                    }
                }
                return(valuesRetVal);
            }

            //********************************************************************************
            //
            /// <summary>
            /// GetControls:                Returns an array of context menu controls 
            ///                             defined in the container
            /// </summary>
            /// <param name="container">    container</param>
            /// <returns>
            /// Array of context menu controls
            /// </returns>
 		    //
            //********************************************************************************
            private object[] GetControls(IContainer container) {
                ArrayList   availableControls = new ArrayList();
                Control     serverControl;

                foreach (IComponent component in container.Components) {
                    serverControl = component as Control;
                    if (serverControl != null) {
                        if (!(serverControl is Page)                &&
                            serverControl.ID != null                &&
                            serverControl.ID.Length > 0             &&
                            IncludeControl(serverControl) == true) {
                            availableControls.Add(serverControl.ID);
                        }
                    }
                }
                availableControls.Sort(Comparer.Default);
                return(availableControls.ToArray());
            }

            //********************************************************************************
            //
            /// <summary>
            /// IncludeControl:                 Check if the control is a context menu
            /// </summary>
            /// <param name="serverControl">    Control</param>
            /// <returns>
            /// True:   The control is a context menu
            /// False:  The control is not a context menu
            /// </returns>
 		    //
            //********************************************************************************
            private bool IncludeControl(Control serverControl) {
                return(serverControl is ContextMenu);
            }
        } // Class MenuControlConverter
    } // Class ContextMenuLink
} // Namespace
