using System;
using System.Text;
using System.Windows.Forms;

namespace RepPP.Toolkit.Window {
    //********************************************************************************
    //
    /// <summary>
    /// Implement the base functionnality of a Rep++ control which inherit from a Forms control and
    /// supports fields property.
    /// Because .NET doesn't support multiple inheritance, this class must be embedded
    /// </summary>
    //
    //********************************************************************************
    public class WndFldControlUtil : RepPP.Toolkit.FldControlUtil {
        private RepPP.FieldSelector m_fieldSelector;

        //********************************************************************************
        //
        /// <summary>
        /// ctor:                           Class constuctor
        /// </summary>
        /// <param name="parent">           Control embedding this class</param>
        /// <param name="bResDispenserOnly">Must only use the Resource Dispenser to create the object</param>
        //
        //********************************************************************************
        public WndFldControlUtil(System.Windows.Forms.Control parent, bool bResDispenserOnly) : base(parent, bResDispenserOnly) {
            m_fieldSelector     = new RepPP.FieldSelector(parent);
            parent.GotFocus    += new EventHandler(parent_GotFocus);
            parent.LostFocus   += new EventHandler(parent_LostFocus);
		}

        //********************************************************************************
        //
        /// <summary>
        /// Dispose:                        Dispose the class
        /// </summary>
        //
        //********************************************************************************
        public override void Dispose() {
            Control ctlParent;

            ctlParent               = Parent as Control;
            ctlParent.GotFocus     -= new EventHandler(parent_GotFocus);
            ctlParent.LostFocus    -= new EventHandler(parent_LostFocus);
            if (m_fieldSelector != null) {
                m_fieldSelector.Dispose();
                m_fieldSelector         = null;
            }
            base.Dispose();
        }

        //********************************************************************************
        //
        /// <summary>
        /// parent_GotFocus:                            Called when the control is getting the focus
        /// </summary>
        /// <param name="sender">                       Sender object</param>
        /// <param name="e">                            Event arguments</param>
        //
        //********************************************************************************
        protected void parent_GotFocus(object sender, EventArgs e) {
            if (!IsDesignMode) {
                m_fieldSelector.Select(Field, true);
            }
        }

        //********************************************************************************
        //
        /// <summary>
        /// parent_LostFocus:                           Called when the control is loosing the focus
        /// </summary>
        /// <param name="sender">                       Sender object</param>
        /// <param name="e">                            Event arguments</param>
        //
        //********************************************************************************
        protected virtual void parent_LostFocus(object sender, EventArgs e) {
            if (IsDesignMode) {
                m_fieldSelector.Select(Field, false);
            }
        }
	} // Class WndFldControlUtil
} // Namespace
