using System;
using System.Collections;
using System.IO;
using Extensibility;
using System.Runtime.InteropServices;
#if VC8
using Microsoft.VisualStudio.CommandBars;
#else
using Microsoft.Office.Core;
#endif
using EnvDTE;
using RepPP.VSNetWizard;

namespace RepPP.VSAddIn {
    [GuidAttribute(Global.ADDIN_GUID), ProgIdAttribute(Global.ADDIN_PRGID)]
	public class RepPPVSAddIn : Extensibility.IDTExtensibility2, IDTCommandTarget {
		private _DTE            m_dte;
		private AddIn           m_addInInstance;
		private const string    m_strCmdName1     = "UpdateTypedInstances";
		private const string    m_strFullCmdName1 = Global.ADDIN_PRGID + "." + m_strCmdName1;
		private const string    m_strCmdText1     = "Update Existing Rep++ Typed Instances";
		private const string    m_strCmdName2     = "SyncAllTypedInstances";
		private const string    m_strFullCmdName2 = Global.ADDIN_PRGID + "." + m_strCmdName2;
		private const string    m_strCmdText2     = "Synchronize All Rep++ Types";

		public RepPPVSAddIn()	{
		    m_dte           = null;
		    m_addInInstance = null;
		}
		
		public void OnConnection(object application, Extensibility.ext_ConnectMode connectMode, object addInInst, ref System.Array custom) {
            Command     cmdObj;
            CommandBar  cmdBar;
            object[]    arrObj;
			
			m_dte               = (_DTE)application;
			m_addInInstance     = (AddIn)addInInst;
			arrObj              = new object[0];
            if (connectMode == Extensibility.ext_ConnectMode.ext_cm_Startup ||
                connectMode == Extensibility.ext_ConnectMode.ext_cm_AfterStartup) {
                try {
                    // Add UpdateTypedInstances command
                    try {
                        cmdObj  = m_dte.Commands.Item(m_strFullCmdName1, -1);
                    } catch {
                        cmdObj = null;
                    }
                    if (cmdObj != null) {
                        cmdObj.Delete();
                        cmdObj = null;
                    }
                    cmdObj  = m_dte.Commands.AddNamedCommand((EnvDTE.AddIn)m_addInInstance,
                                                             m_strCmdName1,
                                                             m_strCmdText1,
                                                             m_strCmdText1,
                                                             true,
                                                             0,
                                                             ref arrObj,
                                                             1+2);
                    cmdBar = ((CommandBars)m_dte.CommandBars)["Folder"];
                    cmdObj.AddControl(cmdBar, 1);
                    // Add SyncAllTypedInstances command
                    try {
                        cmdObj  = m_dte.Commands.Item(m_strFullCmdName2, -1);
                    } catch {
                        cmdObj = null;
                    }
                    if (cmdObj != null) {
                        cmdObj.Delete();
                        cmdObj = null;
                    }
                    cmdObj  = m_dte.Commands.AddNamedCommand((EnvDTE.AddIn)m_addInInstance,
                                                             m_strCmdName2,
                                                             m_strCmdText2,
                                                             m_strCmdText2,
                                                             true,
                                                             0,
                                                             ref arrObj,
                                                             1+2);
                    cmdBar = ((CommandBars)m_dte.CommandBars)["Folder"];
                    cmdObj.AddControl(cmdBar, 2);
                } catch (System.Exception ex) {
                    System.Windows.Forms.MessageBox.Show(ex.ToString());
                }
            }
		}
		
		public void OnDisconnection(Extensibility.ext_DisconnectMode disconnectMode, ref System.Array custom) {
		}

		public void OnAddInsUpdate(ref System.Array custom)	{
		}

		public void OnStartupComplete(ref System.Array custom) {
		}

		public void OnBeginShutdown(ref System.Array custom) {
		}

        private ProjectItem GetTypedDirectory() {
            ProjectItem     itemRetVal = null;
            ProjectItem     projItem;
            SelectedItem    itemSelected;
            
            if (m_dte.SelectedItems.Count == 1) {
                itemSelected = m_dte.SelectedItems.Item(1) as SelectedItem;
                if (itemSelected != null) {
                    projItem = itemSelected.ProjectItem;
                    if (projItem != null &&
                        projItem.FileCount == 1 &&
                        projItem.Name == "TypedInstances") {
                        itemRetVal = projItem;
                    }
                }
            }
            return(itemRetVal);
        }
        
        public void Exec(string strCmdName, vsCommandExecOption eExecuteOption, ref object objVarIn, ref object objVarOut, ref bool bHandled) {
            ProjectItem                         item;
            ProjectFolderInterface              projFolderIntf;
            RepPP.Toolkit.TypedInstanceGenBase  typedGen;
            frmTypedInstNameClashLog            frmLog;

            bHandled = false;
            if (eExecuteOption == vsCommandExecOption.vsCommandExecOptionDoDefault) {
                if (strCmdName == m_strFullCmdName1 ||
                    strCmdName == m_strFullCmdName2) {
                    bHandled = true;
                    item     = GetTypedDirectory();
                    if (item != null) {
                        switch(item.ContainingProject.Kind) {
                        case VSLangProj.PrjKind.prjKindCSharpProject:
                            typedGen = new RepPP.Toolkit.TypedInstanceGenForCS();
                            break;
                        case VSLangProj.PrjKind.prjKindVBProject:
                            typedGen = new RepPP.Toolkit.TypedInstanceGenForVB();
                            break;
                        default:
                            typedGen = null;
                            break;
                        }
                        if (typedGen != null) {
                            projFolderIntf = new ProjectFolderInterface(item);
                            try {
                                using (frmLog = new frmTypedInstNameClashLog()) {
                                    typedGen.RegenerateTypedInstance(frmLog,
                                                                     projFolderIntf,
                                                                     (strCmdName == m_strFullCmdName2));
                                    if (frmLog.MessageText != String.Empty) {
                                        frmLog.ShowDialog();
                                    }
                                }
                            } catch(System.Exception exc) {
                                System.Windows.Forms.MessageBox.Show(exc.Message);
                            }
                        }
                    }
                }
            }
        }

        public void QueryStatus(string strCmdName, vsCommandStatusTextWanted eNeededText, ref vsCommandStatus eStatusOption, ref object commandText) {
            ProjectItem     item;
            
            if (eNeededText == EnvDTE.vsCommandStatusTextWanted.vsCommandStatusTextWantedNone) {
                if (strCmdName == m_strFullCmdName1 ||
                    strCmdName == m_strFullCmdName2) {
                    eStatusOption = vsCommandStatus.vsCommandStatusSupported;
                    item            = GetTypedDirectory();
                    eStatusOption   = vsCommandStatus.vsCommandStatusSupported;
                    eStatusOption  |= (item == null) ?
                                      vsCommandStatus.vsCommandStatusInvisible :
                                      vsCommandStatus.vsCommandStatusEnabled;
                } else {
                    eStatusOption = vsCommandStatus.vsCommandStatusUnsupported;
                }
            }
        }
        
	} // Class RepPPVSAddIn
}
