using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;

namespace RepPP.VSNetWizard {
    //******************************************************************
    //
	/// <summary>
	/// frmStepRepConnect: Enter the typed instances generation parameters
	/// </summary>
    //
    //******************************************************************
	internal class frmStepTypedInstParam : frmStepBase {
        protected System.Windows.Forms.CheckBox         chkGenWSAdapters;
        protected System.Windows.Forms.Label            lblNamespace;
        protected System.Windows.Forms.TextBox          txtNamespace;
        protected System.Windows.Forms.TextBox          txtCardBaseClass;
        protected System.Windows.Forms.Label            lblCardBaseClass;
        protected System.Windows.Forms.Label            lblGroupBaseClass;
        protected System.Windows.Forms.CheckBox         chkGenTypedInst;
        protected System.Windows.Forms.TextBox          txtGroupBaseClass;
        private System.Windows.Forms.GroupBox           grpPrefix;
        protected System.Windows.Forms.TextBox          txtPrefixCardInst;
        protected System.Windows.Forms.Label            lblPrefixCardInst;
        protected System.Windows.Forms.Label            lblPrefixGroupInst;
        protected System.Windows.Forms.TextBox          txtPrefixGroupInst;
        protected System.Windows.Forms.Label            lblPrefixFldProp;
        protected System.Windows.Forms.TextBox          txtPrefixFldProp;
        protected System.Windows.Forms.Label            lblPrefixGroupInstProp;
        protected System.Windows.Forms.TextBox          txtPrefixGroupInstProp;
		private System.ComponentModel.Container         components = null;

        //******************************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //******************************************************************
		public frmStepTypedInstParam() {
			InitializeComponent();
            chkGenTypedInst.Checked     = true;
            chkGenWSAdapters.Checked    = true;
		}

        //******************************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        /// <param name="frmMain">      Main form</param>
        //
        //******************************************************************
		public frmStepTypedInstParam(frmMainBase frmMain) : base(frmMain) {
			InitializeComponent();
            chkGenTypedInst.Checked     = true;
            chkGenWSAdapters.Checked    = true;
            InitParameters(m_frmMain.CurrentProject);
		}

        //******************************************************************
        //
        /// <summary>
        /// Dispose:                    Dispose the form
        /// </summary>
        /// <param name="bDisposing">   true if disposing, false if finalizing</param>
        //
        //******************************************************************
        protected override void Dispose(bool bDisposing) {
            if (bDisposing) {
                if (components != null) {
                    components.Dispose();
                }
            }
            base.Dispose(bDisposing);
        }

        //******************************************************************
        //
        /// <summary>
        /// InitParameters:             Initialize the generation parameters
        /// </summary>
        /// <param name="project">      The current project</param>
        //
        //******************************************************************
        protected void InitParameters(EnvDTE.Project project) {
		    string              strSysName;
		    string              strAppName;
		    string              strConName;
		    string              strUsrName;
		    LanguageCode        eLangCode;
		    string              strNameSpace;
            string              strRootNameSpace;
		    string              strCardBaseClass;
		    string              strGroupBaseClass;
		    bool                bGenWS;
            string              strCrdClassPrefix;
            string              strGrpClassPrefix;
            string              strGrpPropertyPrefix;
            string              strFldPropertyPrefix;

            if (project != null) {
                Utility.ExtractConfigInfo(project,
		                                  out strSysName,
		                                  out strAppName,
		                                  out strConName,
		                                  out strUsrName,
		                                  out eLangCode,
                                          out strCrdClassPrefix,
                                          out strGrpClassPrefix,
                                          out strGrpPropertyPrefix,
                                          out strFldPropertyPrefix,
		                                  out strNameSpace,
                                          out strRootNameSpace,
		                                  out strCardBaseClass,
		                                  out strGroupBaseClass,
		                                  out bGenWS);
                txtNamespace.Text           = strNameSpace;
                txtCardBaseClass.Text       = strCardBaseClass;
                txtGroupBaseClass.Text      = strGroupBaseClass;
                txtPrefixCardInst.Text      = strCrdClassPrefix;
                txtPrefixGroupInst.Text     = strGrpClassPrefix;
                txtPrefixGroupInstProp.Text = strGrpPropertyPrefix;
                txtPrefixFldProp.Text       = strFldPropertyPrefix;
            }
            if (txtCardBaseClass.Text == String.Empty) {
                txtCardBaseClass.Text  = Utility.GetRegistryKeyValue(Global.TYPEDINST_WIZARD_REGKEY, "CardBaseClass");
            }
            if (txtGroupBaseClass.Text == String.Empty) {
                txtGroupBaseClass.Text = Utility.GetRegistryKeyValue(Global.TYPEDINST_WIZARD_REGKEY, "GroupBaseClass");
            }
            if (txtPrefixCardInst.Text == String.Empty) {
                txtPrefixCardInst.Text  = Utility.GetRegistryKeyValue(Global.TYPEDINST_WIZARD_REGKEY, "CardInstClassPrefix");
            }
            if (txtPrefixGroupInst.Text == String.Empty) {
                txtPrefixGroupInst.Text  = Utility.GetRegistryKeyValue(Global.TYPEDINST_WIZARD_REGKEY, "GroupInstClassPrefix");
            }
            if (txtPrefixGroupInstProp.Text == String.Empty) {
                txtPrefixGroupInstProp.Text  = Utility.GetRegistryKeyValue(Global.TYPEDINST_WIZARD_REGKEY, "GroupInstPropertyPrefix");
            }
            if (txtPrefixFldProp.Text == String.Empty) {
                txtPrefixFldProp.Text  = Utility.GetRegistryKeyValue(Global.TYPEDINST_WIZARD_REGKEY, "FieldPropertyPrefix");
            }

            if (txtNamespace.Text == String.Empty                     &&
                m_frmMain != null                                     && 
                m_frmMain.ProgramLanguage == ProgramLanguageE.sdCSharp) {
                if (project != null) {
                    txtNamespace.Text = Utility.GetProjectRootNamespace(project);
                } else {
                    // In ASP.NET 2.0 the project name can be the directory path name. In this case we extract the name
                    // of the directory to consider it the default namespace.
                    txtNamespace.Text = System.IO.Path.GetFileName(System.IO.Path.GetDirectoryName(m_frmMain.ProjectName));
                }
            }
            if (txtCardBaseClass.Text == String.Empty) {
                txtCardBaseClass.Text   = "RepPP.TypedCardInstanceBase";
            }
            if (txtGroupBaseClass.Text == String.Empty) {
                txtGroupBaseClass.Text  = "RepPP.TypedGroupInstanceBase";
            }
            if (txtPrefixCardInst.Text == String.Empty) {
                txtPrefixCardInst.Text  = "CC";
            }
            if (txtPrefixGroupInst.Text == String.Empty) {
                txtPrefixGroupInst.Text  = "GC";
            }
            if (txtPrefixGroupInstProp.Text == String.Empty) {
                txtPrefixGroupInstProp.Text  = "gi";
            }
            if (txtPrefixFldProp.Text == String.Empty) {
                txtPrefixFldProp.Text  = "fld";
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// GenTypedInstance:             Generate typed instance flag
        /// </summary>
        //
        //******************************************************************
        public Utility.GenTypedInstTypeE GenTypedInstance {
            get {
                Utility.GenTypedInstTypeE eRetVal;

                if (chkGenTypedInst.Checked) {
                    if (chkGenWSAdapters.Checked) {
                        eRetVal = Utility.GenTypedInstTypeE.TypedInstAndWSAdapter;
                    } else {
                        eRetVal = Utility.GenTypedInstTypeE.TypedInstOnly;
                    }
                } else {
                    eRetVal = Utility.GenTypedInstTypeE.None;
                }
                return(eRetVal);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// Namespace:             Typed instances namespace
        /// </summary>
        //
        //******************************************************************
        public string Namespace {
            get {
                return(txtNamespace.Text);
            }
            set {
                txtNamespace.Text = value;
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// CardInstanceBaseClass:             Base class for card typed instances
        /// </summary>
        //
        //******************************************************************
        public string CardInstanceBaseClass {
            get {
                return(txtCardBaseClass.Text);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// CardInstanceBaseClass:             Base class for group typed instances
        /// </summary>
        //
        //******************************************************************
        public string GroupInstanceBaseClass {
            get {
                return(txtGroupBaseClass.Text);
            }
        } 

        //******************************************************************
        //
        /// <summary>
        /// CardInstanceClassPrefix:    Prefix for the typed card instance class name
        /// </summary>
        //
        //******************************************************************
        public string CardInstanceClassPrefix {
            get {
                return(txtPrefixCardInst.Text);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// GroupInstanceClassPrefix:    Prefix for the typed group instance class name
        /// </summary>
        //
        //******************************************************************
        public string GroupInstanceClassPrefix {
            get {
                return(txtPrefixGroupInst.Text);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// GroupInstancePropertyPrefix:    Prefix for the typed group instance property name
        /// </summary>
        //
        //******************************************************************
        public string GroupInstancePropertyPrefix {
            get {
                return(txtPrefixGroupInstProp.Text);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// FieldPropertyPrefix:        Prefix for the field property name
        /// </summary>
        //
        //******************************************************************
        public string FieldPropertyPrefix {
            get {
                return(txtPrefixFldProp.Text);
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// ValidateB:                    Validate the step
        /// </summary>
        /// <returns>
        /// True:   Valid
        /// False:  Not Valid
        /// </returns>
        //
        //******************************************************************
        public override bool ValidateB() {
            bool bRetVal;

            if (chkGenTypedInst.Checked) {
                bRetVal = (txtCardBaseClass.Text  != String.Empty) &&
                          (txtGroupBaseClass.Text != String.Empty);
            } else {
                bRetVal = true;
            }
            return(bRetVal);
        }

        //******************************************************************
        //
        /// <summary>
        /// SaveTempFolder:            Save the last template folder used 
        /// </summary>
        //
        //******************************************************************
        public void SaveTypedInstParam() {
            Utility.SetRegistryKeyValue(Global.TYPEDINST_WIZARD_REGKEY, "CardBaseClass",  txtCardBaseClass.Text);
            Utility.SetRegistryKeyValue(Global.TYPEDINST_WIZARD_REGKEY, "GroupBaseClass", txtGroupBaseClass.Text);
            Utility.SetRegistryKeyValue(Global.TYPEDINST_WIZARD_REGKEY, "CardInstClassPrefix", txtPrefixCardInst.Text);
            Utility.SetRegistryKeyValue(Global.TYPEDINST_WIZARD_REGKEY, "GroupInstClassPrefix", txtPrefixGroupInst.Text);
            Utility.SetRegistryKeyValue(Global.TYPEDINST_WIZARD_REGKEY, "GroupInstPropertyPrefix", txtPrefixGroupInstProp.Text);
            Utility.SetRegistryKeyValue(Global.TYPEDINST_WIZARD_REGKEY, "FieldPropertyPrefix", txtPrefixFldProp.Text);
        }       

        //*********************************************************     
        //
        /// <summary>
        /// OnTypedInstChecked:      Handle the check event on the typed instances
        /// </summary>
        /// <param name="sender">    Sender object   </param>
        /// <param name="e">         Event Arguments </param>
        //  
        //*********************************************************     
        protected virtual void OnTypedInstChecked(object sender, System.EventArgs e) {
            chkGenWSAdapters.Enabled    = chkGenTypedInst.Checked;
            txtNamespace.Enabled        = chkGenTypedInst.Checked;
            txtCardBaseClass.Enabled    = chkGenTypedInst.Checked;
            txtGroupBaseClass.Enabled   = chkGenTypedInst.Checked;
            if (m_frmMain != null) {
                m_frmMain.NextButton.Enabled = ValidateB();
            }
        }

        //******************************************************************
        //
        /// <summary>
        /// OnParamChanged:         Handle changed event on the one of the parameters
        /// </summary>
        /// <param name="sender">   Sender</param>
        /// <param name="e">        Event arguments</param>
        //
        //******************************************************************
        protected virtual void OnParamChanged(object sender, System.EventArgs e) {
            if (m_frmMain != null) {
                m_frmMain.NextButton.Enabled = ValidateB();
            }
        }
		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.chkGenWSAdapters = new System.Windows.Forms.CheckBox();
            this.lblNamespace = new System.Windows.Forms.Label();
            this.txtNamespace = new System.Windows.Forms.TextBox();
            this.txtCardBaseClass = new System.Windows.Forms.TextBox();
            this.lblCardBaseClass = new System.Windows.Forms.Label();
            this.txtGroupBaseClass = new System.Windows.Forms.TextBox();
            this.lblGroupBaseClass = new System.Windows.Forms.Label();
            this.chkGenTypedInst = new System.Windows.Forms.CheckBox();
            this.grpPrefix = new System.Windows.Forms.GroupBox();
            this.txtPrefixCardInst = new System.Windows.Forms.TextBox();
            this.lblPrefixCardInst = new System.Windows.Forms.Label();
            this.lblPrefixGroupInst = new System.Windows.Forms.Label();
            this.txtPrefixGroupInst = new System.Windows.Forms.TextBox();
            this.lblPrefixFldProp = new System.Windows.Forms.Label();
            this.txtPrefixFldProp = new System.Windows.Forms.TextBox();
            this.lblPrefixGroupInstProp = new System.Windows.Forms.Label();
            this.txtPrefixGroupInstProp = new System.Windows.Forms.TextBox();
            this.pnlStep.SuspendLayout();
            this.grpPrefix.SuspendLayout();
            this.SuspendLayout();
            // 
            // pnlStep
            // 
            this.pnlStep.Controls.Add(this.grpPrefix);
            this.pnlStep.Controls.Add(this.chkGenTypedInst);
            this.pnlStep.Controls.Add(this.txtGroupBaseClass);
            this.pnlStep.Controls.Add(this.txtCardBaseClass);
            this.pnlStep.Controls.Add(this.txtNamespace);
            this.pnlStep.Controls.Add(this.lblGroupBaseClass);
            this.pnlStep.Controls.Add(this.lblCardBaseClass);
            this.pnlStep.Controls.Add(this.lblNamespace);
            this.pnlStep.Controls.Add(this.chkGenWSAdapters);
            this.pnlStep.Name = "pnlStep";
            this.pnlStep.Size = new System.Drawing.Size(416, 280);
            // 
            // chkGenWSAdapters
            // 
            this.chkGenWSAdapters.Location = new System.Drawing.Point(200, 8);
            this.chkGenWSAdapters.Name = "chkGenWSAdapters";
            this.chkGenWSAdapters.Size = new System.Drawing.Size(192, 24);
            this.chkGenWSAdapters.TabIndex = 0;
            this.chkGenWSAdapters.Text = "Generate Web Services Adapters";
            // 
            // lblNamespace
            // 
            this.lblNamespace.Location = new System.Drawing.Point(16, 40);
            this.lblNamespace.Name = "lblNamespace";
            this.lblNamespace.Size = new System.Drawing.Size(120, 23);
            this.lblNamespace.TabIndex = 1;
            this.lblNamespace.Text = "Namespace";
            // 
            // txtNamespace
            // 
            this.txtNamespace.Location = new System.Drawing.Point(160, 40);
            this.txtNamespace.Name = "txtNamespace";
            this.txtNamespace.Size = new System.Drawing.Size(232, 20);
            this.txtNamespace.TabIndex = 2;
            this.txtNamespace.Text = "";
            this.txtNamespace.TextChanged += new System.EventHandler(this.OnParamChanged);
            // 
            // txtCardBaseClass
            // 
            this.txtCardBaseClass.Location = new System.Drawing.Point(160, 72);
            this.txtCardBaseClass.Name = "txtCardBaseClass";
            this.txtCardBaseClass.Size = new System.Drawing.Size(232, 20);
            this.txtCardBaseClass.TabIndex = 4;
            this.txtCardBaseClass.Text = "";
            this.txtCardBaseClass.TextChanged += new System.EventHandler(this.OnParamChanged);
            // 
            // lblCardBaseClass
            // 
            this.lblCardBaseClass.Location = new System.Drawing.Point(16, 72);
            this.lblCardBaseClass.Name = "lblCardBaseClass";
            this.lblCardBaseClass.Size = new System.Drawing.Size(136, 23);
            this.lblCardBaseClass.TabIndex = 3;
            this.lblCardBaseClass.Text = "Card Instance Base Class";
            // 
            // txtGroupBaseClass
            // 
            this.txtGroupBaseClass.Location = new System.Drawing.Point(160, 104);
            this.txtGroupBaseClass.Name = "txtGroupBaseClass";
            this.txtGroupBaseClass.Size = new System.Drawing.Size(232, 20);
            this.txtGroupBaseClass.TabIndex = 6;
            this.txtGroupBaseClass.Text = "";
            this.txtGroupBaseClass.TextChanged += new System.EventHandler(this.OnParamChanged);
            // 
            // lblGroupBaseClass
            // 
            this.lblGroupBaseClass.Location = new System.Drawing.Point(16, 104);
            this.lblGroupBaseClass.Name = "lblGroupBaseClass";
            this.lblGroupBaseClass.Size = new System.Drawing.Size(144, 23);
            this.lblGroupBaseClass.TabIndex = 5;
            this.lblGroupBaseClass.Text = "Group Instance Base Class";
            // 
            // chkGenTypedInst
            // 
            this.chkGenTypedInst.Location = new System.Drawing.Point(16, 8);
            this.chkGenTypedInst.Name = "chkGenTypedInst";
            this.chkGenTypedInst.Size = new System.Drawing.Size(168, 24);
            this.chkGenTypedInst.TabIndex = 7;
            this.chkGenTypedInst.Text = "Generate Typed Instances";
            this.chkGenTypedInst.CheckedChanged += new System.EventHandler(this.OnTypedInstChecked);
            // 
            // grpPrefix
            // 
            this.grpPrefix.Controls.Add(this.lblPrefixFldProp);
            this.grpPrefix.Controls.Add(this.txtPrefixFldProp);
            this.grpPrefix.Controls.Add(this.lblPrefixGroupInstProp);
            this.grpPrefix.Controls.Add(this.txtPrefixGroupInstProp);
            this.grpPrefix.Controls.Add(this.lblPrefixGroupInst);
            this.grpPrefix.Controls.Add(this.txtPrefixGroupInst);
            this.grpPrefix.Controls.Add(this.lblPrefixCardInst);
            this.grpPrefix.Controls.Add(this.txtPrefixCardInst);
            this.grpPrefix.Location = new System.Drawing.Point(16, 128);
            this.grpPrefix.Name = "grpPrefix";
            this.grpPrefix.Size = new System.Drawing.Size(392, 144);
            this.grpPrefix.TabIndex = 8;
            this.grpPrefix.TabStop = false;
            this.grpPrefix.Text = "Prefix";
            // 
            // txtPrefixCardInst
            // 
            this.txtPrefixCardInst.Location = new System.Drawing.Point(144, 16);
            this.txtPrefixCardInst.Name = "txtPrefixCardInst";
            this.txtPrefixCardInst.Size = new System.Drawing.Size(232, 20);
            this.txtPrefixCardInst.TabIndex = 3;
            this.txtPrefixCardInst.Text = "";
            // 
            // lblPrefixCardInst
            // 
            this.lblPrefixCardInst.Location = new System.Drawing.Point(8, 16);
            this.lblPrefixCardInst.Name = "lblPrefixCardInst";
            this.lblPrefixCardInst.Size = new System.Drawing.Size(136, 23);
            this.lblPrefixCardInst.TabIndex = 4;
            this.lblPrefixCardInst.Text = "Card Instance Class";
            // 
            // lblPrefixGroupInst
            // 
            this.lblPrefixGroupInst.Location = new System.Drawing.Point(8, 48);
            this.lblPrefixGroupInst.Name = "lblPrefixGroupInst";
            this.lblPrefixGroupInst.Size = new System.Drawing.Size(136, 23);
            this.lblPrefixGroupInst.TabIndex = 6;
            this.lblPrefixGroupInst.Text = "Group Instance Class";
            // 
            // txtPrefixGroupInst
            // 
            this.txtPrefixGroupInst.Location = new System.Drawing.Point(144, 46);
            this.txtPrefixGroupInst.Name = "txtPrefixGroupInst";
            this.txtPrefixGroupInst.Size = new System.Drawing.Size(232, 20);
            this.txtPrefixGroupInst.TabIndex = 5;
            this.txtPrefixGroupInst.Text = "";
            // 
            // lblPrefixFldProp
            // 
            this.lblPrefixFldProp.Location = new System.Drawing.Point(8, 112);
            this.lblPrefixFldProp.Name = "lblPrefixFldProp";
            this.lblPrefixFldProp.Size = new System.Drawing.Size(136, 23);
            this.lblPrefixFldProp.TabIndex = 10;
            this.lblPrefixFldProp.Text = "Field Property";
            // 
            // txtPrefixFldProp
            // 
            this.txtPrefixFldProp.Location = new System.Drawing.Point(144, 110);
            this.txtPrefixFldProp.Name = "txtPrefixFldProp";
            this.txtPrefixFldProp.Size = new System.Drawing.Size(232, 20);
            this.txtPrefixFldProp.TabIndex = 9;
            this.txtPrefixFldProp.Text = "";
            // 
            // lblPrefixGroupInstProp
            // 
            this.lblPrefixGroupInstProp.Location = new System.Drawing.Point(8, 80);
            this.lblPrefixGroupInstProp.Name = "lblPrefixGroupInstProp";
            this.lblPrefixGroupInstProp.Size = new System.Drawing.Size(136, 23);
            this.lblPrefixGroupInstProp.TabIndex = 8;
            this.lblPrefixGroupInstProp.Text = "Group Instance Property";
            // 
            // txtPrefixGroupInstProp
            // 
            this.txtPrefixGroupInstProp.Location = new System.Drawing.Point(144, 78);
            this.txtPrefixGroupInstProp.Name = "txtPrefixGroupInstProp";
            this.txtPrefixGroupInstProp.Size = new System.Drawing.Size(232, 20);
            this.txtPrefixGroupInstProp.TabIndex = 7;
            this.txtPrefixGroupInstProp.Text = "";
            // 
            // frmStepTypedInstParam
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(448, 302);
            this.Name = "frmStepTypedInstParam";
            this.Text = "frmStepTypedInstParam";
            this.pnlStep.ResumeLayout(false);
            this.grpPrefix.ResumeLayout(false);
            this.ResumeLayout(false);

        }
		#endregion
	} // Class frmStepTypedInstParam
} // Namespace
