//*********************************************************
//
// frmMain:     Main form in which the step forms will be attached
//
// Author:		Maher Najah
//
// (c) 2004 by Consyst SQL Inc.
//
//
//*********************************************************
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Xml;
using System.IO;
using System.Web;
using Microsoft.Win32;
using RepPP.VSNetWizard;
using RepPP.Toolkit;

namespace RepPP.VSNetWizard.TypedInst
{
	/// <summary>
	/// Main wizard form.
	/// </summary>
	internal class frmMain : frmMainBase {
        private frmStep0                            m_frmStep0;             // step 0 form
        private frmStep1                            m_frmStep1;             // step 1 form
        private frmStep2                            m_frmStep2;             // step 2 form
        private frmStep3                            m_frmStep3;             // step 3 form
        private frmStep4                            m_frmStep4;             // step 4 form
        private string                              m_strTypedInstSubDir;   // Typed instances subdirectory
                   
		private System.ComponentModel.Container     components = null;

        //******************************************************************
        //
        /// <summary>
        /// ctor:                       Class constructor
        /// </summary>
        //
        //******************************************************************
		public frmMain() {
			InitializeComponent();
		}

        //******************************************************************
        //
        /// <summary>
        /// ctor:                           Class constructor
        /// </summary>
        /// <param name="vsApp">            Visual studio application</param>
        /// <param name="strPrjName">       Project name</param>
        /// <param name="strPrjFullName">   Project full name</param>
        /// <param name="eLang">            Language</param>
        /// <param name="strSubDirectory">  Subdirectory</param>
        //
        //******************************************************************
		public frmMain(EnvDTE.DTE       vsApp,
                       string           strPrjName,
                       string           strPrjFullName,
                       ProgramLanguageE eLang,
                       string           strSubDirectory) : base (vsApp,
                                                                 strPrjName,
                                                                 strPrjFullName,
                                                                 eLang,
                                                                 GenerationModeE.sdItem) {
			InitializeComponent();
            m_frmStep0                = new frmStep0(this);
            m_frmStep1                = new frmStep1(this);
            m_frmStep2                = new frmStep2(this);
            m_frmStep3                = new frmStep3(this);
            m_frmStep4                = new frmStep4(this);
            m_frmStep0.AttachToMainPanelV();
            butBack.Enabled           = false;
            butNext.Enabled           = true;
            m_strTypedInstSubDir      = strSubDirectory;
		}

        //******************************************************************
        //
        /// <summary>
        /// ctor:                           Class constructor
        /// </summary>
        /// <param name="vsApp">            Visual studio application</param>
        /// <param name="strPrjName">       Project name</param>
        /// <param name="strPrjFullName">   Project full name</param>
        /// <param name="eLang">            Language</param>
        //
        //******************************************************************
		public frmMain(EnvDTE.DTE       vsApp,
                       string           strPrjName,
                       string           strPrjFullName,
                       ProgramLanguageE eLang) : this(vsApp,
                                                      strPrjName,
                                                      strPrjFullName,
                                                      eLang,
                                                      string.Empty) {
		}

        //******************************************************************
        //
        /// <summary>
        /// Dispose:                    Dispose the form
        /// </summary>
        /// <param name="bDisposing">   true if disposing, false if finalizing</param>
        //
        //******************************************************************
        protected override void Dispose(bool bDisposing) {
            if (bDisposing) {
                if (components != null) {
                    components.Dispose();
                }
                if (m_frmStep0 != null) {
                    m_frmStep0.Dispose();
                    m_frmStep0 = null;
                }
                if (m_frmStep1 != null) {
                    m_frmStep1.Dispose();
                    m_frmStep1 = null;
                }
                if (m_frmStep2 != null) {
                    m_frmStep2.Dispose();
                    m_frmStep2 = null;
                }
                if (m_frmStep3 != null) {
                    m_frmStep3.Dispose();
                    m_frmStep3 = null;
                }
                if (m_frmStep4 != null) {
                    m_frmStep4.Dispose();
                    m_frmStep4 = null;
                }
            }
            base.Dispose(bDisposing);
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnButNextClick:         Handle next button click event
        /// </summary>
        /// <param name="sender">   Sender object   </param>
        /// <param name="e">        Event Arguments </param>
        //  
        //*********************************************************     
        protected override void OnButNextClick(object sender, System.EventArgs e) {
            switch(m_iCurStep) {
            case 0:
                SetStepPanelV(m_frmStep1, true);
                break;
            case 1:
                if (InitializeRepPP(m_frmStep1.System.ToUpper(),
                                    m_frmStep1.Program.ToUpper(),
                                    m_frmStep1.Connection.ToUpper(),
                                    m_frmStep1.User.ToUpper(),
                                    m_frmStep1.Password,
                                    (RepPP.LanguageCode)m_frmStep1.Language)) {
                    SetStepPanelV(m_frmStep2, true);
                } else {
                    m_iCurStep--;
                }
                break;
            case 2:
                SetStepPanelV(m_frmStep3, true);
                break;
            case 3:
                if (Utility.GetProjectRootNamespace(m_vsApp, m_strPrjName) == String.Empty &&
                    m_frmStep3.Namespace == String.Empty) {
                    MessageBox.Show("The project does not have a root or a default namespace and no namespace is specified for the typed instances. \r\nPlease specify a namespace for the typed instances.",
                                    "Rep++ Typed Instances Wizard",
                                    MessageBoxButtons.OK,
                                    MessageBoxIcon.Error);
                    m_iCurStep--;
                } else if (!Utility.CheckForTypedInstNameClash(m_app,
                                                               m_frmStep2.SelectedCards,
                                                               m_frmStep2.SelectedGroups, 
                                                               m_frmStep3.CardInstanceBaseClass,
                                                               m_frmStep3.GroupInstanceBaseClass,
                                                               m_frmStep3.CardInstanceClassPrefix,
                                                               m_frmStep3.GroupInstanceClassPrefix,
                                                               m_frmStep3.GroupInstancePropertyPrefix,
                                                               m_frmStep3.FieldPropertyPrefix,
                                                               m_frmStep3.GenTypedInstance == Utility.GenTypedInstTypeE.TypedInstAndWSAdapter)) {
                    m_iCurStep--;
                } else {
                    SetStepPanelV(m_frmStep4, true);
                    butNext.Enabled      = false;
                    butCancelFinish.Text = "&Finish";
                }
                break;
            }
            m_iCurStep++;
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnButBackClick:         Handle back button click event
        /// </summary>
        /// <param name="sender">   Sender object   </param>
        /// <param name="e">        Event Arguments </param>
        //  
        //*********************************************************     
        protected override void OnButBackClick(object sender, System.EventArgs e) {
            switch(m_iCurStep) {
            case 1:
                SetStepPanelV(m_frmStep0, false);
                butBack.Enabled = false;
                break;
            case 2:
                SetStepPanelV(m_frmStep1, false);
                break;
            case 3:
                SetStepPanelV(m_frmStep2, false);
                break;
            case 4:
                SetStepPanelV(m_frmStep3, false);
                butCancelFinish.Text = "&Cancel";
                break;
            }
            m_iCurStep--;        
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnButCancelFinishClick:     Handle cancel/finish button click event
        /// </summary>
        //  
        //*********************************************************     
        public override void Generate() {
            TypedInstanceGenBase    generator = null;
            bool                    bGenWSAdapters;
            EnvDTE.Project          project;
            string                  strNamespace;
            string                  strRootNamespace;

            project          = Utility.GetProject(m_vsApp, m_strPrjName);
            bGenWSAdapters   = (m_frmStep3.GenTypedInstance == Utility.GenTypedInstTypeE.TypedInstAndWSAdapter);            
            strNamespace     = m_frmStep3.Namespace;
            strRootNamespace = String.Empty;
            if (strNamespace == String.Empty) {
                if (ProgramLanguage == ProgramLanguageE.sdVisualBasic) {
                    strRootNamespace = Utility.GetProjectRootNamespace(m_vsApp, m_strPrjName);
                } else {
                    strNamespace = Utility.GetProjectRootNamespace(m_vsApp, m_strPrjName);
                }
                if (strNamespace == String.Empty && strRootNamespace == String.Empty) {
                    strNamespace = m_strPrjName;
                }
            }
            switch (m_eLang) {
            case ProgramLanguageE.sdCSharp:
                Utility.AddAssemblyAttribute(project, "RepPP.AssemblyIncludeTypes", ".cs");
                generator = new TypedInstanceGenForCS();
                break;
            case ProgramLanguageE.sdVisualBasic:
                Utility.AddAssemblyAttribute(project, "RepPP.AssemblyIncludeTypes", ".vb");
                generator = new TypedInstanceGenForVB();
                break;
            default:
                break;
            }
            foreach (string strCardName in m_frmStep2.SelectedCards) {
                Utility.GenCardTypedInstance(project,
                                             m_frmStep3.CardInstanceClassPrefix,
                                             m_frmStep3.GroupInstanceClassPrefix,
                                             m_frmStep3.GroupInstancePropertyPrefix,
                                             m_frmStep3.FieldPropertyPrefix,
                                             strNamespace,
                                             strRootNamespace,
                                             m_frmStep3.CardInstanceBaseClass,
                                             m_frmStep3.GroupInstanceBaseClass,
                                             generator,
                                             bGenWSAdapters,
                                             m_app.Cards[strCardName],
                                             m_frmStep4.ReplaceFileMode,
                                             m_strTypedInstSubDir,
                                             false);
            }
            foreach(string strGroupName in m_frmStep2.SelectedGroups) {
                Utility.GenGroupTypedInstance(project,
                                              m_frmStep3.CardInstanceClassPrefix,
                                              m_frmStep3.GroupInstanceClassPrefix,
                                              m_frmStep3.GroupInstancePropertyPrefix,
                                              m_frmStep3.FieldPropertyPrefix,
                                              strNamespace,
                                              strRootNamespace,
                                              m_frmStep3.GroupInstanceBaseClass,
                                              generator,
                                              bGenWSAdapters,
                                              m_app.Groups[strGroupName],
                                              m_frmStep4.ReplaceFileMode,
                                              m_strTypedInstSubDir);
            }
            Utility.GenTypedInstanceConfigFile(project,
                                               m_frmStep1.System.ToUpper(),
                                               m_frmStep1.Program.ToUpper(),
                                               m_frmStep1.Connection.ToUpper(),
                                               m_frmStep1.User.ToUpper(),
                                               (RepPP.LanguageCode)m_frmStep1.Language,
                                               m_frmStep3.CardInstanceClassPrefix,
                                               m_frmStep3.GroupInstanceClassPrefix,
                                               m_frmStep3.GroupInstancePropertyPrefix,
                                               m_frmStep3.FieldPropertyPrefix,
                                               strNamespace,
                                               strRootNamespace,
                                               m_frmStep3.CardInstanceBaseClass,
                                               m_frmStep3.GroupInstanceBaseClass,
                                               bGenWSAdapters);
            m_frmStep3.SaveTypedInstParam();
        }

        //*********************************************************     
        //
        /// <summary>
        /// OnButCancelFinishClick:     Handle the clicke event on finixh button
        /// </summary>
        /// <param name="sender">       Sender object   </param>
        /// <param name="e">            Event Arguments </param>
        //  
        //*********************************************************     
        protected override void OnButCancelFinishClick(object sender, System.EventArgs e) {
            try {
                if (butCancelFinish.Text == "&Finish") {
#if !VC8
                    Generate();
#endif
                    this.DialogResult = System.Windows.Forms.DialogResult.OK;
                } else {
                    this.DialogResult = System.Windows.Forms.DialogResult.Cancel;
                }
            } catch (System.Exception ex) {
                MessageBox.Show(ex.ToString());
            } finally {
                Close();
            }
        }
		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(frmMain));
            this.SuspendLayout();
            // 
            // pnlMain
            // 
            this.pnlMain.Name = "pnlMain";
            // 
            // butCancelFinish
            // 
            this.butCancelFinish.Name = "butCancelFinish";
            // 
            // butNext
            // 
            this.butNext.Name = "butNext";
            // 
            // butBack
            // 
            this.butBack.Name = "butBack";
            // 
            // pictureBox1
            // 
            this.pictureBox1.BackgroundImage = ((System.Drawing.Image)(resources.GetObject("pictureBox1.BackgroundImage")));
            this.pictureBox1.Name = "pictureBox1";
            // 
            // grpBar
            // 
            this.grpBar.Name = "grpBar";
            // 
            // frmMain
            // 
            this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
            this.ClientSize = new System.Drawing.Size(554, 326);
            this.Icon = ((System.Drawing.Icon)(resources.GetObject("$this.Icon")));
            this.Name = "frmMain";
            this.Text = "Rep++ Typed Instance Wizard";
            this.ResumeLayout(false);

        }
		#endregion
	} // class frmMain
} // namespace
